#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# "Hello Friends" in Chinese: 朋友你好
import os
import sys
import time
import functools
from argparse import ArgumentParser
import counter.counter
import sitesize.sitesize


def timeit(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        s = time.time()
        r = f(*args, **kwargs)
        print(f'{f.__name__} took {time.time() - s:1.3f}s.')
        return r
    return wrapper


def set_v_print(verbose: bool):
    """
    Defines the function v_print.
    It prints if verbose is true, otherwise, it does nothing.
    See: http://stackoverflow.com/questions/5980042
    :param verbose: A bool to determine if v_print will print its args.
    """
    global v_print
    v_print = print if verbose else lambda *a, **k: None


class Coffee:

    def __init__(self, price: float):
        self._price = price

    @property
    def price(self):
        return self._price

    @price.setter
    def price(self, new_price: float):
        if new_price > 0 and isinstance(new_price, float):
            self._price = new_price
        else:
            raise ValueError("price must be a non-negative float.")


@timeit
def main(debug: bool):
    start_time = time.time()
    script_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
    v_print("Running counter...")
    counter.counter.run()
    v_print("Running sitesize...")
    sitesize.sitesize.run()
    cuppa = Coffee(5.00)
    cuppa.price = cuppa.price - 1.00
    print(f'{sys.argv[0]} is in {script_dir}.')


if __name__ == '__main__':
    parser = ArgumentParser(description='Just a template sample.')
    parser.add_argument('-d', '--debug', action='store_true')
    parser.add_argument('-v', '--verbose', action='store_true')
    args = parser.parse_args()
    set_v_print(args.verbose)
    main(args.debug)
